@@ -140,7 +140,7 @@ CDecoder::CDecoder(bool multiThread)
   multiThread = true;
   #endif
   _multiThread = multiThread;
-  _bindInfoExPrevIsDefinded = false;
+  _bindInfoExPrevIsDefined = false;
   #ifndef EXCLUDE_COM
   LoadMethodMap();
   #endif
@@ -155,6 +155,9 @@ HRESULT CDecoder::Decode(IInStream *inStream,
     #ifndef _NO_CRYPTO
     , ICryptoGetTextPassword *getTextPassword
     #endif
+    #ifdef COMPRESS_MT
+    , bool mtMode, UInt32 numThreads
+    #endif
     )
 {
   CObjectVector< CMyComPtr<ISequentialInStream> > inStreams;
@@ -173,7 +176,8 @@ HRESULT CDecoder::Decode(IInStream *inStream,
     CLimitedSequentialInStream *streamSpec = new 
         CLimitedSequentialInStream;
     CMyComPtr<ISequentialInStream> inStream = streamSpec;
-    streamSpec->Init(lockedStreamImp, packSizes[j]);
+    streamSpec->SetStream(lockedStreamImp);
+    streamSpec->Init(packSizes[j]);
     inStreams.Add(inStream);
   }
   
@@ -182,7 +186,7 @@ HRESULT CDecoder::Decode(IInStream *inStream,
   CBindInfoEx bindInfo;
   ConvertFolderItemInfoToBindInfo(folderInfo, bindInfo);
   bool createNewCoders;
-  if (!_bindInfoExPrevIsDefinded)
+  if (!_bindInfoExPrevIsDefined)
     createNewCoders = true;
   else
     createNewCoders = !AreBindInfoExEqual(bindInfo, _bindInfoExPrev);
@@ -316,7 +320,7 @@ HRESULT CDecoder::Decode(IInStream *inStream,
       }
     }
     _bindInfoExPrev = bindInfo;
-    _bindInfoExPrevIsDefinded = true;
+    _bindInfoExPrevIsDefined = true;
   }
   int i;
   _mixerCoderCommon->ReInit();
@@ -329,50 +333,60 @@ HRESULT CDecoder::Decode(IInStream *inStream,
   {
     const CCoderInfo &coderInfo = folderInfo.Coders[i];
     const CAltCoderInfo &altCoderInfo = coderInfo.AltCoders.Front();
-    CMyComPtr<ICompressSetDecoderProperties2> compressSetDecoderProperties;
-    HRESULT result = _decoders[coderIndex].QueryInterface(
-        IID_ICompressSetDecoderProperties2, &compressSetDecoderProperties);
+    CMyComPtr<IUnknown> &decoder = _decoders[coderIndex];
     
-    if (result == S_OK)
     {
-      const CByteBuffer &properties = altCoderInfo.Properties;
-      size_t size = properties.GetCapacity();
-      if (size > 0xFFFFFFFF)
-        return E_NOTIMPL;
-      if (size > 0)
+      CMyComPtr<ICompressSetDecoderProperties2> setDecoderProperties;
+      decoder.QueryInterface(IID_ICompressSetDecoderProperties2, &setDecoderProperties);
+      if (setDecoderProperties)
       {
-        RINOK(compressSetDecoderProperties->SetDecoderProperties2((const Byte *)properties, (UInt32)size));
+        const CByteBuffer &properties = altCoderInfo.Properties;
+        size_t size = properties.GetCapacity();
+        if (size > 0xFFFFFFFF)
+          return E_NOTIMPL;
+        if (size > 0)
+        {
+          RINOK(setDecoderProperties->SetDecoderProperties2((const Byte *)properties, (UInt32)size));
+        }
       }
     }
-    else if (result != E_NOINTERFACE)
-      return result;
+
+    #ifdef COMPRESS_MT
+    if (mtMode)
+    {
+      CMyComPtr<ICompressSetCoderMt> setCoderMt;
+      decoder.QueryInterface(IID_ICompressSetCoderMt, &setCoderMt);
+      if (setCoderMt)
+      {
+        RINOK(setCoderMt->SetNumberOfThreads(numThreads));
+      }
+    }
+    #endif
 
     #ifndef _NO_CRYPTO
-    CMyComPtr<ICryptoSetPassword> cryptoSetPassword;
-    result = _decoders[coderIndex].QueryInterface(
-        IID_ICryptoSetPassword, &cryptoSetPassword);
-
-    if (result == S_OK)
     {
-      if (getTextPassword == 0)
-        return E_FAIL;
-      CMyComBSTR password;
-      RINOK(getTextPassword->CryptoGetTextPassword(&password));
-      CByteBuffer buffer;
-      UString unicodePassword(password);
-      const UInt32 sizeInBytes = unicodePassword.Length() * 2;
-      buffer.SetCapacity(sizeInBytes);
-      for (int i = 0; i < unicodePassword.Length(); i++)
+      CMyComPtr<ICryptoSetPassword> cryptoSetPassword;
+      decoder.QueryInterface(IID_ICryptoSetPassword, &cryptoSetPassword);
+      if (cryptoSetPassword)
       {
-        wchar_t c = unicodePassword[i];
-        ((Byte *)buffer)[i * 2] = (Byte)c;
-        ((Byte *)buffer)[i * 2 + 1] = (Byte)(c >> 8);
+        if (getTextPassword == 0)
+          return E_FAIL;
+        CMyComBSTR password;
+        RINOK(getTextPassword->CryptoGetTextPassword(&password));
+        CByteBuffer buffer;
+        UString unicodePassword(password);
+        const UInt32 sizeInBytes = unicodePassword.Length() * 2;
+        buffer.SetCapacity(sizeInBytes);
+        for (int i = 0; i < unicodePassword.Length(); i++)
+        {
+          wchar_t c = unicodePassword[i];
+          ((Byte *)buffer)[i * 2] = (Byte)c;
+          ((Byte *)buffer)[i * 2 + 1] = (Byte)(c >> 8);
+        }
+        RINOK(cryptoSetPassword->CryptoSetPassword(
+          (const Byte *)buffer, sizeInBytes));
       }
-      RINOK(cryptoSetPassword->CryptoSetPassword(
-        (const Byte *)buffer, sizeInBytes));
     }
-    else if (result != E_NOINTERFACE)
-      return result;
     #endif
 
     coderIndex++;
