@@ -8,12 +8,14 @@
 #include "7zMethods.h"
 
 #include "../../../Windows/PropVariant.h"
+
 #include "../../../Common/ComTry.h"
 #include "../../../Common/StringToInt.h"
 #include "../../IPassword.h"
 #include "../../ICoder.h"
 
 #include "../Common/ItemNameUtils.h"
+#include "../Common/ParseProperties.h"
 
 using namespace NWindows;
 
@@ -22,6 +24,7 @@ namespace N7z {
 
 #ifdef COMPRESS_LZMA
 static CMethodID k_LZMA = { { 0x3, 0x1, 0x1 }, 3 };
+static CMethodID k_LZMA2 = { { 0x3, 0x1, 0x2 }, 3 };
 #endif
 
 #ifdef COMPRESS_PPMD
@@ -62,35 +65,85 @@ static CMethodID k_BZip2 = { { 0x4, 0x2, 0x2 }, 3 };
 
 const wchar_t *kCopyMethod = L"Copy";
 const wchar_t *kLZMAMethodName = L"LZMA";
+const wchar_t *kLZMA2MethodName = L"LZMA2";
 const wchar_t *kBZip2MethodName = L"BZip2";
+const wchar_t *kPpmdMethodName = L"PPMd";
+const wchar_t *kDeflateMethodName = L"Deflate";
+const wchar_t *kDeflate64MethodName = L"Deflate64";
 
-const UInt32 kAlgorithmForX7 = 2;
-const UInt32 kDicSizeForX7 = 1 << 23;
-const UInt32 kFastBytesForX7 = 64;
+static const wchar_t *kLzmaMatchFinderX1 = L"HC4";
+static const wchar_t *kLzmaMatchFinderX5 = L"BT4";
 
-const UInt32 kAlgorithmForX9 = 2;
-const UInt32 kDicSizeForX9 = 1 << 25;
-const UInt32 kFastBytesForX9 = 64;
-static const wchar_t *kMatchFinderForX9 = L"BT4b";
+static const UInt32 kLzmaAlgorithmX1 = 0;
+static const UInt32 kLzmaAlgorithmX5 = 1;
 
-const UInt32 kAlgorithmForFast = 0;
-const UInt32 kDicSizeForFast = 1 << 15;
-static const wchar_t *kMatchFinderForFast = L"HC3";
+static const UInt32 kLzmaDicSizeX1 = 1 << 16;
+static const UInt32 kLzmaDicSizeX3 = 1 << 20;
+static const UInt32 kLzmaDicSizeX5 = 1 << 22;
+static const UInt32 kLzmaDicSizeX7 = 1 << 24;
+static const UInt32 kLzmaDicSizeX9 = 1 << 26;
+
+static const UInt32 kLzmaFastBytesX1 = 32;
+static const UInt32 kLzmaFastBytesX7 = 64;
+
+static const UInt32 kPpmdMemSizeX1 = (1 << 22);
+static const UInt32 kPpmdMemSizeX5 = (1 << 24);
+static const UInt32 kPpmdMemSizeX7 = (1 << 26);
+static const UInt32 kPpmdMemSizeX9 = (192 << 20);
+
+static const UInt32 kPpmdOrderX1 = 4;
+static const UInt32 kPpmdOrderX5 = 6;
+static const UInt32 kPpmdOrderX7 = 16;
+static const UInt32 kPpmdOrderX9 = 32;
+
+static const UInt32 kDeflateFastBytesX1 = 32;
+static const UInt32 kDeflateFastBytesX7 = 64;
+static const UInt32 kDeflateFastBytesX9 = 128;
+
+static const UInt32 kDeflatePassesX1 = 1;
+static const UInt32 kDeflatePassesX7 = 3;
+static const UInt32 kDeflatePassesX9 = 10;
+
+static const UInt32 kBZip2NumPassesX1 = 1;
+static const UInt32 kBZip2NumPassesX7 = 2;
+static const UInt32 kBZip2NumPassesX9 = 7;
+
+static const UInt32 kBZip2DicSizeX1 = 100000;
+static const UInt32 kBZip2DicSizeX3 = 500000;
+static const UInt32 kBZip2DicSizeX5 = 900000;
 
 const wchar_t *kDefaultMethodName = kLZMAMethodName;
 
-static const wchar_t *kMatchFinderForHeaders = L"BT2";
+static const wchar_t *kLzmaMatchFinderForHeaders = L"BT2";
 static const UInt32 kDictionaryForHeaders = 1 << 20;
 static const UInt32 kNumFastBytesForHeaders = 273;
+static const UInt32 kAlgorithmForHeaders = kLzmaAlgorithmX5;
+
+static bool IsCopyMethod(const UString &methodName)
+  { return (methodName.CompareNoCase(kCopyMethod) == 0); }
 
 static bool IsLZMAMethod(const UString &methodName)
-  { return (methodName.CompareNoCase(kLZMAMethodName) == 0); }
+{ 
+  return 
+  (methodName.CompareNoCase(kLZMAMethodName) == 0) || 
+  (methodName.CompareNoCase(kLZMA2MethodName) == 0); 
+}
+
+/*
 static bool IsLZMethod(const UString &methodName)
   { return IsLZMAMethod(methodName); }
+*/
 
 static bool IsBZip2Method(const UString &methodName)
   { return (methodName.CompareNoCase(kBZip2MethodName) == 0); }
 
+static bool IsPpmdMethod(const UString &methodName)
+  { return (methodName.CompareNoCase(kPpmdMethodName) == 0); }
+
+static bool IsDeflateMethod(const UString &methodName)
+  { return (methodName.CompareNoCase(kDeflateMethodName) == 0) || 
+  (methodName.CompareNoCase(kDeflate64MethodName) == 0); }
+
 STDMETHODIMP CHandler::GetFileTimeType(UInt32 *type)
 {
   *type = NFileTimeType::kWindows;
@@ -113,7 +166,8 @@ HRESULT CHandler::SetPassword(CCompressionMethodMode &methodMode,
     Int32 passwordIsDefined;
     RINOK(getTextPassword->CryptoGetTextPassword2(
         &passwordIsDefined, &password));
-    if (methodMode.PasswordIsDefined = IntToBool(passwordIsDefined))
+    methodMode.PasswordIsDefined = IntToBool(passwordIsDefined);
+    if (methodMode.PasswordIsDefined)
       methodMode.Password = password;
   }
   else
@@ -137,9 +191,10 @@ CNameToPropID g_NameToPropID[] =
 
   { NCoderPropID::kNumPasses, VT_UI4, L"Pass" },
   { NCoderPropID::kNumFastBytes, VT_UI4, L"fb" },
+  { NCoderPropID::kMatchFinderCycles, VT_UI4, L"mc" },
   { NCoderPropID::kAlgorithm, VT_UI4, L"a" },
   { NCoderPropID::kMatchFinder, VT_BSTR, L"mf" },
-  { NCoderPropID::kMultiThread, VT_BOOL, L"mt" }
+  { NCoderPropID::kNumThreads, VT_UI4, L"mt" }
 };
 
 bool ConvertProperty(PROPVARIANT srcProp, VARTYPE varType, 
@@ -178,7 +233,12 @@ HRESULT CHandler::SetCompressionMethod(
     CCompressionMethodMode &methodMode,
     CCompressionMethodMode &headerMethod)
 {
-  RINOK(SetCompressionMethod(methodMode, _methods, _multiThread));
+  HRESULT res = SetCompressionMethod(methodMode, _methods
+  #ifdef COMPRESS_MT
+  , _numThreads
+  #endif
+  );
+  RINOK(res);
   methodMode.Binds = _binds;
   if (_compressHeadersFull)
     _compressHeaders = true;
@@ -193,13 +253,13 @@ HRESULT CHandler::SetCompressionMethod(
     {
       CProperty property;
       property.PropID = NCoderPropID::kMatchFinder;
-      property.Value = kMatchFinderForHeaders;
+      property.Value = kLzmaMatchFinderForHeaders;
       oneMethodInfo.CoderProperties.Add(property);
     }
     {
       CProperty property;
       property.PropID = NCoderPropID::kAlgorithm;
-      property.Value = kAlgorithmForX9;
+      property.Value = kAlgorithmForHeaders;
       oneMethodInfo.CoderProperties.Add(property);
     }
     {
@@ -215,7 +275,12 @@ HRESULT CHandler::SetCompressionMethod(
       oneMethodInfo.CoderProperties.Add(property);
     }
     headerMethodInfoVector.Add(oneMethodInfo);
-    RINOK(SetCompressionMethod(headerMethod, headerMethodInfoVector, false));
+    HRESULT res = SetCompressionMethod(headerMethod, headerMethodInfoVector
+      #ifdef COMPRESS_MT
+      ,1
+      #endif
+    );
+    RINOK(res);
   }
   return S_OK;
 }
@@ -237,8 +302,11 @@ static void SetOneMethodProp(COneMethodInfo &oneMethodInfo, PROPID propID,
 
 HRESULT CHandler::SetCompressionMethod(
     CCompressionMethodMode &methodMode,
-    CObjectVector<COneMethodInfo> &methodsInfo,
-    bool multiThread)
+    CObjectVector<COneMethodInfo> &methodsInfo
+    #ifdef COMPRESS_MT
+    , UInt32 numThreads
+    #endif
+    )
 {
   #ifndef EXCLUDE_COM
   /*
@@ -249,40 +317,103 @@ HRESULT CHandler::SetCompressionMethod(
   #endif
  
 
+  UInt32 level = _level;
+  
   if (methodsInfo.IsEmpty())
   {
     COneMethodInfo oneMethodInfo;
-    oneMethodInfo.MethodName = _copyMode ? kCopyMethod : kDefaultMethodName;
+    oneMethodInfo.MethodName = ((level == 0) ? kCopyMethod : kDefaultMethodName);
     methodsInfo.Add(oneMethodInfo);
   }
 
+  bool needSolid = false;
   for(int i = 0; i < methodsInfo.Size(); i++)
   {
     COneMethodInfo &oneMethodInfo = methodsInfo[i];
     if (oneMethodInfo.MethodName.IsEmpty())
       oneMethodInfo.MethodName = kDefaultMethodName;
 
-    if (IsLZMethod(oneMethodInfo.MethodName))
+    if (!IsCopyMethod(oneMethodInfo.MethodName))
+      needSolid = true;
+    
+    if (IsLZMAMethod(oneMethodInfo.MethodName))
     {
-      if (IsLZMAMethod(oneMethodInfo.MethodName))
-      {
-        SetOneMethodProp(oneMethodInfo, 
-            NCoderPropID::kDictionarySize, _defaultDicSize);
-        SetOneMethodProp(oneMethodInfo, 
-            NCoderPropID::kAlgorithm, _defaultAlgorithm);
-        SetOneMethodProp(oneMethodInfo, 
-            NCoderPropID::kNumFastBytes, _defaultFastBytes);
-        SetOneMethodProp(oneMethodInfo, 
-            NCoderPropID::kMatchFinder, (const wchar_t *)_defaultMatchFinder);
-        if (multiThread)
-          SetOneMethodProp(oneMethodInfo, 
-              NCoderPropID::kMultiThread, true);
-      }
+      UInt32 dicSize = 
+                  (level >= 9 ? kLzmaDicSizeX9 : 
+                  (level >= 7 ? kLzmaDicSizeX7 : 
+                  (level >= 5 ? kLzmaDicSizeX5 : 
+                  (level >= 3 ? kLzmaDicSizeX3 : 
+                                kLzmaDicSizeX1)))); 
+
+      UInt32 algorithm = 
+                    (level >= 5 ? kLzmaAlgorithmX5 : 
+                                  kLzmaAlgorithmX1); 
+
+      UInt32 fastBytes = 
+                    (level >= 7 ? kLzmaFastBytesX7 : 
+                                  kLzmaFastBytesX1); 
+
+      const wchar_t *matchFinder = 
+                      (level >= 5 ? kLzmaMatchFinderX5 : 
+                                    kLzmaMatchFinderX1); 
+
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kDictionarySize, dicSize);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kAlgorithm, algorithm);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumFastBytes, fastBytes);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kMatchFinder, matchFinder);
+      #ifdef COMPRESS_MT
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumThreads, numThreads);
+      #endif
+    }
+    else if (IsDeflateMethod(oneMethodInfo.MethodName))
+    {
+      UInt32 fastBytes = 
+                    (level >= 9 ? kDeflateFastBytesX9 : 
+                    (level >= 7 ? kDeflateFastBytesX7 : 
+                                  kDeflateFastBytesX1));
+
+      UInt32 numPasses = 
+                    (level >= 9 ? kDeflatePassesX9 :  
+                    (level >= 7 ? kDeflatePassesX7 : 
+                                  kDeflatePassesX1));
+
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumFastBytes, fastBytes);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumPasses, numPasses);
     }
     else if (IsBZip2Method(oneMethodInfo.MethodName))
     {
-      SetOneMethodProp(oneMethodInfo, 
-        NCoderPropID::kNumPasses, _defaultBZip2Passes);
+      UInt32 numPasses = 
+                    (level >= 9 ? kBZip2NumPassesX9 : 
+                    (level >= 7 ? kBZip2NumPassesX7 :  
+                                  kBZip2NumPassesX1));
+
+      UInt32 dicSize = 
+                  (level >= 5 ? kBZip2DicSizeX5 : 
+                  (level >= 3 ? kBZip2DicSizeX3 : 
+                                kBZip2DicSizeX1));
+
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumPasses, numPasses);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kDictionarySize, dicSize);
+      #ifdef COMPRESS_MT
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kNumThreads, numThreads);
+      #endif
+    }
+    else if (IsPpmdMethod(oneMethodInfo.MethodName))
+    {
+      UInt32 useMemSize = 
+                     (level >= 9 ? kPpmdMemSizeX9 : 
+                     (level >= 7 ? kPpmdMemSizeX7 : 
+                     (level >= 5 ? kPpmdMemSizeX5 : 
+                                   kPpmdMemSizeX1)));
+
+      UInt32 order = 
+                (level >= 9 ? kPpmdOrderX9 : 
+                (level >= 7 ? kPpmdOrderX7 : 
+                (level >= 5 ? kPpmdOrderX5 : 
+                              kPpmdOrderX1)));
+
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kUsedMemorySize, useMemSize);
+      SetOneMethodProp(oneMethodInfo, NCoderPropID::kOrder, order);
     }
 
 
@@ -383,7 +514,46 @@ HRESULT CHandler::SetCompressionMethod(
       return E_FAIL;
     
     methodMode.Methods.Add(methodFull);
+
+    if (!_numSolidBytesDefined)
+    {
+      for (int j = 0; j < methodFull.CoderProperties.Size(); j++)
+      {
+        const CProperty &prop = methodFull.CoderProperties[j];
+        if ((prop.PropID == NCoderPropID::kDictionarySize || 
+             prop.PropID == NCoderPropID::kUsedMemorySize) && prop.Value.vt == VT_UI4)
+        {
+          _numSolidBytes = ((UInt64)prop.Value.ulVal) << 7;
+          const UInt64 kMinSize = (1 << 24);
+          if (_numSolidBytes < kMinSize)
+            _numSolidBytes = kMinSize;
+          _numSolidBytesDefined = true;
+          break;
+        }
+      }
+    }
   }
+
+  if (!needSolid && !_numSolidBytesDefined)
+  {
+    _numSolidBytesDefined = true;
+    _numSolidBytes  = 0;
+  }
+  return S_OK;
+}
+
+static HRESULT GetTime(IArchiveUpdateCallback *updateCallback, int index, PROPID propID, CArchiveFileTime &filetime, bool &filetimeIsDefined)
+{
+  filetimeIsDefined = false;
+  NCOM::CPropVariant propVariant;
+  RINOK(updateCallback->GetProperty(index, propID, &propVariant));
+  if (propVariant.vt == VT_FILETIME)
+  {
+    filetime = propVariant.filetime;
+    filetimeIsDefined = true;
+  }
+  else if (propVariant.vt != VT_EMPTY)
+    return E_INVALIDARG;
   return S_OK;
 }
 
@@ -414,7 +584,6 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
   // CMyComPtr<IUpdateCallback2> updateCallback2;
   // updateCallback->QueryInterface(&updateCallback2);
 
-  int index = 0;
   for(UInt32 i = 0; i < numItems; i++)
   {
     Int32 newData;
@@ -439,8 +608,13 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
       updateItem.IsDirectory = fileItem.IsDirectory;
       updateItem.Size = fileItem.UnPackSize;
       updateItem.IsAnti = fileItem.IsAnti;
+      
+      updateItem.CreationTime = fileItem.CreationTime;
+      updateItem.IsCreationTimeDefined = fileItem.IsCreationTimeDefined;
       updateItem.LastWriteTime = fileItem.LastWriteTime;
-      updateItem.LastWriteTimeIsDefined = fileItem.IsLastWriteTimeDefined;
+      updateItem.IsLastWriteTimeDefined = fileItem.IsLastWriteTimeDefined;
+      updateItem.LastAccessTime = fileItem.LastAccessTime;
+      updateItem.IsLastAccessTimeDefined = fileItem.IsLastAccessTimeDefined;
     }
 
     if (updateItem.NewProperties)
@@ -460,32 +634,11 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
           updateItem.AttributesAreDefined = true;
         }
       }
-      {
-        NCOM::CPropVariant propVariant;
-        RINOK(updateCallback->GetProperty(i, kpidCreationTime, &propVariant));
-        if (propVariant.vt == VT_EMPTY)
-          updateItem.CreationTimeIsDefined = false;
-        else if (propVariant.vt != VT_FILETIME)
-          return E_INVALIDARG;
-        else
-        {
-          updateItem.CreationTime = propVariant.filetime;
-          updateItem.CreationTimeIsDefined = true;
-        }
-      }
-      {
-        NCOM::CPropVariant propVariant;
-        RINOK(updateCallback->GetProperty(i, kpidLastWriteTime, &propVariant));
-        if (propVariant.vt == VT_EMPTY)
-          updateItem.LastWriteTimeIsDefined = false;
-        else if (propVariant.vt != VT_FILETIME)
-          return E_INVALIDARG;
-        else
-        {
-          updateItem.LastWriteTime = propVariant.filetime;
-          updateItem.LastWriteTimeIsDefined = true;
-        }
-      }
+      
+      RINOK(GetTime(updateCallback, i, kpidCreationTime, updateItem.CreationTime, updateItem.IsCreationTimeDefined));
+      RINOK(GetTime(updateCallback, i, kpidLastWriteTime, updateItem.LastWriteTime , updateItem.IsLastWriteTimeDefined));
+      RINOK(GetTime(updateCallback, i, kpidLastAccessTime, updateItem.LastAccessTime, updateItem.IsLastAccessTimeDefined));
+
       {
         NCOM::CPropVariant propVariant;
         RINOK(updateCallback->GetProperty(i, kpidPath, &propVariant));
@@ -527,8 +680,11 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
       if (updateItem.IsAnti)
       {
         updateItem.AttributesAreDefined = false;
-        updateItem.CreationTimeIsDefined = false;
-        updateItem.LastWriteTimeIsDefined = false;
+
+        updateItem.IsCreationTimeDefined = false;
+        updateItem.IsLastWriteTimeDefined = false;
+        updateItem.IsLastAccessTimeDefined = false;
+        
         updateItem.Size = 0;
       }
 
@@ -546,35 +702,15 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
       if (updateItem.Size != 0 && updateItem.IsAnti)
         return E_INVALIDARG;
     }
-    /*
-    else
-      thereIsCopyData = true;
-    */
-
     updateItems.Add(updateItem);
   }
 
-  /*
-  if (thereIsCopyData)
-  {
-    for(int i = 0; i < _database.NumUnPackStreamsVector.Size(); i++)
-      if (_database.NumUnPackStreamsVector[i] != 1)
-        return E_NOTIMPL;
-    if (!_solidIsSpecified)
-      _solid = false;
-    if (_solid)
-      return E_NOTIMPL;
-  }
-  */
-
-
   CCompressionMethodMode methodMode, headerMethod;
   RINOK(SetCompressionMethod(methodMode, headerMethod));
-  methodMode.MultiThread = _multiThread;
-  // methodMode.MultiThreadMult = _multiThreadMult;
-
-  headerMethod.MultiThread = false;
-  // headerMethod.MultiThreadMult = _multiThreadMult;
+  #ifdef COMPRESS_MT
+  methodMode.NumThreads = _numThreads;
+  headerMethod.NumThreads = 1;
+  #endif
 
   RINOK(SetPassword(methodMode, updateCallback));
 
@@ -604,8 +740,13 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
       &headerMethod : 0;
   options.UseFilters = _level != 0 && _autoFilter;
   options.MaxFilter = _level >= 8;
-  options.UseAdditionalHeaderStreams = useAdditionalHeaderStreams;
-  options.CompressMainHeader = compressMainHeader;
+
+  options.HeaderOptions.UseAdditionalHeaderStreams = useAdditionalHeaderStreams;
+  options.HeaderOptions.CompressMainHeader = compressMainHeader;
+  options.HeaderOptions.WriteModified = WriteModified;
+  options.HeaderOptions.WriteCreated = WriteCreated;
+  options.HeaderOptions.WriteAccessed = WriteAccessed;
+  
   options.NumSolidFiles = _numSolidFiles;
   options.NumSolidBytes = _numSolidBytes;
   options.SolidExtension = _solidExtension;
@@ -623,124 +764,6 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
   COM_TRY_END
 }
 
-static int ParseStringToUInt32(const UString &srcString, UInt32 &number)
-{
-  const wchar_t *start = srcString;
-  const wchar_t *end;
-  UInt64 number64 = ConvertStringToUInt64(start, &end);
-  if (number64 > 0xFFFFFFFF) 
-  {
-    number = 0;
-    return 0;
-  }
-  number = (UInt32)number64;
-  return (int)(end - start);
-}
-
-static const int kLogarithmicSizeLimit = 32;
-static const char kByteSymbol = 'B';
-static const char kKiloByteSymbol = 'K';
-static const char kMegaByteSymbol = 'M';
-
-HRESULT ParseDictionaryValues(const UString &srcStringSpec, UInt32 &dicSize)
-{
-  UString srcString = srcStringSpec;
-  srcString.MakeUpper();
-
-  const wchar_t *start = srcString;
-  const wchar_t *end;
-  UInt64 number = ConvertStringToUInt64(start, &end);
-  int numDigits = (int)(end - start);
-  if (numDigits == 0 || srcString.Length() > numDigits + 1)
-    return E_INVALIDARG;
-  if (srcString.Length() == numDigits)
-  {
-    if (number >= kLogarithmicSizeLimit)
-      return E_INVALIDARG;
-    dicSize = (UInt32)1 << (int)number;
-    return S_OK;
-  }
-  switch (srcString[numDigits])
-  {
-    case kByteSymbol:
-      if (number >= ((UInt64)1 << kLogarithmicSizeLimit))
-        return E_INVALIDARG;
-      dicSize = (UInt32)number;
-      break;
-    case kKiloByteSymbol:
-      if (number >= ((UInt64)1 << (kLogarithmicSizeLimit - 10)))
-        return E_INVALIDARG;
-      dicSize = UInt32(number << 10);
-      break;
-    case kMegaByteSymbol:
-      if (number >= ((UInt64)1 << (kLogarithmicSizeLimit - 20)))
-        return E_INVALIDARG;
-      dicSize = UInt32(number << 20);
-      break;
-    default:
-      return E_INVALIDARG;
-  }
-  return S_OK;
-}
-
-static inline UINT GetCurrentFileCodePage()
-{
-  return AreFileApisANSI() ? CP_ACP : CP_OEMCP;
-}
-
-static HRESULT SetBoolProperty(bool &dest, const PROPVARIANT &value)
-{
-  switch(value.vt)
-  {
-    case VT_EMPTY:
-      dest = true;
-      break;
-    /*
-    case VT_UI4:
-      dest = (value.ulVal != 0);
-      break;
-    */
-    case VT_BSTR:
-    {
-      UString valueString = value.bstrVal;
-      valueString.MakeUpper();
-      if (valueString.Compare(L"ON") == 0)
-        dest = true;
-      else if (valueString.Compare(L"OFF") == 0)
-        dest = false;
-      else
-        return E_INVALIDARG;
-      break;
-    }
-    default:
-      return E_INVALIDARG;
-  }
-  return S_OK;
-}
-
-/*
-static HRESULT SetComplexProperty(bool &boolStatus, UInt32 &number, 
-    const PROPVARIANT &value)
-{
-  switch(value.vt)
-  {
-    case VT_EMPTY:
-    case VT_BSTR:
-    {
-      RINOK(SetBoolProperty(boolStatus, value));
-      return S_OK;
-    }
-    case VT_UI4:
-      boolStatus = true;
-      number = (value.ulVal);
-      break;
-    default:
-      return E_INVALIDARG;
-  }
-  return S_OK;
-}
-*/
-
 static HRESULT GetBindInfoPart(UString &srcString, UInt32 &coder, UInt32 &stream)
 {
   stream = 0;
@@ -820,7 +843,7 @@ HRESULT CHandler::SetParam(COneMethodInfo &oneMethodInfo, const UString &name, c
   if (name.CompareNoCase(L"D") == 0 || name.CompareNoCase(L"MEM") == 0)
   {
     UInt32 dicSize;
-    RINOK(ParseDictionaryValues(value, dicSize));
+    RINOK(ParsePropDictionaryValue(value, dicSize));
     if (name.CompareNoCase(L"D") == 0)
       property.PropID = NCoderPropID::kDictionarySize;
     else
@@ -951,11 +974,17 @@ HRESULT CHandler::SetSolidSettings(const PROPVARIANT &value)
 
 STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *values, Int32 numProperties)
 {
-  UINT codePage = GetCurrentFileCodePage();
   COM_TRY_BEGIN
   _methods.Clear();
   _binds.Clear();
   Init();
+  #ifdef COMPRESS_MT
+  const UInt32 numProcessors = NSystem::GetNumberOfProcessors();
+  #endif
+
+  UInt32 mainDicSize = 0xFFFFFFFF;
+  UInt32 mainDicMethodIndex = 0xFFFFFFFF;
+
   UInt32 minNumber = 0;
 
   for (int i = 0; i < numProperties; i++)
@@ -971,55 +1000,7 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
     {
       name.Delete(0);
       _level = 9;
-      if (value.vt == VT_UI4)
-      {
-        if (!name.IsEmpty())
-          return E_INVALIDARG;
-        _level = value.ulVal;
-      }
-      else if (value.vt == VT_EMPTY)
-      {
-        if(!name.IsEmpty())
-        {
-          int index = ParseStringToUInt32(name, _level);
-          if (index != name.Length())
-            return E_INVALIDARG;
-        }
-      }
-      else
-        return E_INVALIDARG;
-      if (_level == 0)
-      {
-        _copyMode = true;
-        _defaultBZip2Passes = 1;
-      }
-      else if (_level < 5)
-      {
-        _defaultAlgorithm = kAlgorithmForFast;
-        _defaultDicSize = kDicSizeForFast;
-        _defaultMatchFinder = kMatchFinderForFast;
-        _defaultBZip2Passes = 1;
-      }
-      else if (_level < 7)
-      {
-        _defaultBZip2Passes = 1;
-        // normal;
-      }
-      else if(_level < 9)
-      {
-        _defaultAlgorithm = kAlgorithmForX7;
-        _defaultDicSize = kDicSizeForX7;
-        _defaultFastBytes = kFastBytesForX7;
-        _defaultBZip2Passes = 2;
-      }
-      else
-      {
-        _defaultAlgorithm = kAlgorithmForX9;
-        _defaultDicSize = kDicSizeForX9;
-        _defaultFastBytes = kFastBytesForX9;
-        _defaultMatchFinder = kMatchFinderForX9;
-        _defaultBZip2Passes = 7;
-      }
+      RINOK(ParsePropValue(name, value, _level));
       continue;
     }
 
@@ -1052,7 +1033,14 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
     UString realName = name.Mid(index);
     if (index == 0)
     {
-      if (name.CompareNoCase(L"RSFX") == 0)
+      if(name.Left(2).CompareNoCase(L"MT") == 0)
+      {
+        #ifdef COMPRESS_MT
+        RINOK(ParseMtProp(name.Mid(2), value, numProcessors, _numThreads));
+        #endif
+        continue;
+      }
+      else if (name.CompareNoCase(L"RSFX") == 0)
       {
         RINOK(SetBoolProperty(_removeSfxBlock, value));
         continue;
@@ -1077,11 +1065,19 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
         RINOK(SetBoolProperty(_encryptHeaders, value));
         continue;
       }
-      else if (name.CompareNoCase(L"MT") == 0)
+      else if (name.CompareNoCase(L"TM") == 0)
       {
-        // _multiThreadMult = 200;
-        RINOK(SetBoolProperty(_multiThread, value));
-        // RINOK(SetComplexProperty(MultiThread, _multiThreadMult, value));
+        RINOK(SetBoolProperty(WriteModified, value));
+        continue;
+      }
+      else if (name.CompareNoCase(L"TC") == 0)
+      {
+        RINOK(SetBoolProperty(WriteCreated, value));
+        continue;
+      }
+      else if (name.CompareNoCase(L"TA") == 0)
+      {
+        RINOK(SetBoolProperty(WriteAccessed, value));
         continue;
       }
       else if (name.CompareNoCase(L"V") == 0)
@@ -1115,28 +1111,25 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
     else
     {
       CProperty property;
-      if (realName.CompareNoCase(L"D") == 0 || realName.CompareNoCase(L"MEM") == 0)
+      if (realName.Left(1).CompareNoCase(L"D") == 0)
       {
         UInt32 dicSize;
-        if (value.vt == VT_UI4)
-        {
-          UInt32 logDicSize = value.ulVal;
-          if (logDicSize >= 32)
-            return E_INVALIDARG;
-          dicSize = (UInt32)1 << logDicSize;
-        }
-        else if (value.vt == VT_BSTR)
-        {
-          RINOK(ParseDictionaryValues(value.bstrVal, dicSize));
-        }
-        else 
-          return E_FAIL;
-        if (realName.CompareNoCase(L"D") == 0)
-          property.PropID = NCoderPropID::kDictionarySize;
-        else
-          property.PropID = NCoderPropID::kUsedMemorySize;
+        RINOK(ParsePropDictionaryValue(realName.Mid(1), value, dicSize));
+        property.PropID = NCoderPropID::kDictionarySize;
         property.Value = dicSize;
         oneMethodInfo.CoderProperties.Add(property);
+        if (number <= mainDicMethodIndex)
+          mainDicSize = dicSize;
+      }
+      else if (realName.Left(3).CompareNoCase(L"MEM") == 0)
+      {
+        UInt32 dicSize;
+        RINOK(ParsePropDictionaryValue(realName.Mid(3), value, dicSize));
+        property.PropID = NCoderPropID::kUsedMemorySize;
+        property.Value = dicSize;
+        oneMethodInfo.CoderProperties.Add(property);
+        if (number <= mainDicMethodIndex)
+          mainDicSize = dicSize;
       }
       else
       {
@@ -1154,7 +1147,6 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
       }
     }
   }
-  CheckAndSetSolidBytesLimit();
 
   return S_OK;
   COM_TRY_END
