@@ -62,10 +62,10 @@ static UString GetModuleFolderPrefix()
 }
 
 static wchar_t *kFormatFolderName = L"Formats";
-static LPCTSTR kRegistryPath = TEXT("Software\\7-zip");
-static LPCWSTR kProgramPathValue = L"Path";
 
 #ifdef _WIN32
+static LPCTSTR kRegistryPath = TEXT("Software\\7-zip");
+static LPCWSTR kProgramPathValue = L"Path";
 static bool ReadPathFromRegistry(HKEY baseKey, UString &path)
 {
   NRegistry::CKey key;
@@ -104,7 +104,7 @@ typedef UInt32 (WINAPI *CreateObjectPointer)(
 #endif
 
 #ifndef _SFX
-static void SetBuffer(CByteBuffer &bb, const Byte *data, int size)
+static inline void SetBuffer(CByteBuffer &bb, const Byte *data, int size)
 {
   bb.SetCapacity(size);
   memmove((Byte *)bb, data, size);
@@ -244,7 +244,7 @@ void ReadArchiverInfoList(CObjectVector<CArchiverInfo> &archivers)
   #else
 
   UString folderPath = GetBaseFolderPrefixFromRegistry() + 
-      kFormatFolderName + WSTRING_PATH_SEPARATOR;
+      (UString)kFormatFolderName + (UString)WSTRING_PATH_SEPARATOR;
   NFind::CEnumeratorW enumerator(folderPath + L"*");
   NFind::CFileInfoW fileInfo;
   while (enumerator.Next(fileInfo))
