@@ -1,6 +1,10 @@
 
 #if !defined(__DJGPP__)
 
+#ifndef __CYGWIN__
+  #define FILESYSTEM_IS_CASE_SENSITIVE 1
+#endif
+
   #if !defined(ENV_MACOSX) && !defined(ENV_BEOS)
 
     /* <wchar.h> */
@@ -17,7 +21,11 @@
 
   #endif /* !ENV_MACOSX && !ENV_BEOS */
 
-  /* lstat and readlink */
+  #if !defined(ENV_BEOS)
+  #define HAVE_GETPASS
+  #endif
+
+  /* lstat, readlink and S_ISLNK */
   #define HAVE_LSTAT
 
   /* <locale.h> */
@@ -29,11 +37,6 @@
   /* wcstombs */
   #define HAVE_WCSTOMBS
 
-  #if !defined(__CYGWIN__) && !defined(sparc) && !defined(sun) && !defined(__APPLE_CC__) && !defined(ENV_BEOS)
-  /* timegm */
-  #define HAVE_TIMEGM
-  #endif
-
 #endif /* !__DJGPP__ */
 
 #ifndef ENV_BEOS
